package com.tzld.piaoquan.tencentad.controller;

import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.enums.*;
import com.tzld.piaoquan.tencentad.model.vo.*;
import com.tzld.piaoquan.tencentad.service.AdMuseTaskService;
import com.tzld.piaoquan.tencentad.service.AdVideoImageService;
import com.tzld.piaoquan.tencentad.service.AdVideoService;
import com.tzld.piaoquan.tencentad.utils.page.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/ad/video")
public class AdVideoController {

    @Autowired
    private AdVideoService adVideoService;

    @Autowired
    private AdVideoImageService adVideoImageService;

    @Autowired
    private AdMuseTaskService adMuseTaskService;


    @GetMapping("/get")
    public CommonResponse<Page<AdVideoVo>> getAdVideoList(@RequestParam(defaultValue = "1") Integer pageNum,
                                                          @RequestParam(defaultValue = "10") Integer pageSize) {
        return adVideoService.getAdVideoList(pageNum, pageSize);
    }

//    @GetMapping("/status")
//    public CommonResponse<List<AdVideoStatusVo>> getAdVideoStatusVo() {
//        List<AdVideoStatusVo> res = new ArrayList<>();
//        res.add(new AdVideoStatusVo(AdVideoStatusEnum.VERIFY_SUCCESS.getStatus(),
//                AdVideoStatusEnum.VERIFY_SUCCESS.getDesc()));
//        res.add(new AdVideoStatusVo(AdVideoStatusEnum.VERIFY_FAIL.getStatus(),
//                AdVideoStatusEnum.VERIFY_FAIL.getDesc()));
//        return CommonResponse.success(res);
//    }

//    @PostMapping("/update")
//    public CommonResponse<Void> updateAdVideo(@RequestBody AdVideoParam adVideoParam) {
//        return adVideoService.updateAdVideo(adVideoParam);
//    }

    @GetMapping("/image/get")
    public CommonResponse<List<AdVideoImageVo>> getAdVideoImageList(@RequestParam Long adVideoId) {
        return adVideoImageService.getAdVideoImageVoList(adVideoId);
    }

    @GetMapping("/image/status")
    public CommonResponse<List<AdVideoImageStatusVo>> getAdBaseVideoStatusVo() {
        List<AdVideoImageStatusVo> res = new ArrayList<>();
        res.add(new AdVideoImageStatusVo(AdVideoImageStatusEnum.VERIFY_SUCCESS.getStatus(),
                AdVideoImageStatusEnum.VERIFY_SUCCESS.getDesc()));
        res.add(new AdVideoImageStatusVo(AdVideoImageStatusEnum.VERIFY_FAIL.getStatus(),
                AdVideoImageStatusEnum.VERIFY_FAIL.getDesc()));
        return CommonResponse.success(res);
    }

    @GetMapping("/image/dimension")
    public CommonResponse<List<DimensionSizeVo>> getDimensionSizeVo() {
        return CommonResponse.success(Arrays.stream(DimensionSizeEnum.values())
                .map(dimensionSizeEnum -> new DimensionSizeVo(dimensionSizeEnum.getSize(), dimensionSizeEnum.getDesc()))
                .collect(Collectors.toList()));
    }

    @GetMapping("/image/similarity")
    public CommonResponse<List<SimilarityCodeVo>> getSimilarityCodeVo() {
        return CommonResponse.success(Arrays.stream(SimilarityCodeEnum.values())
                .map(similarityCodeEnum -> new SimilarityCodeVo(similarityCodeEnum.getCode(), similarityCodeEnum.getDesc()))
                .collect(Collectors.toList()));
    }

    @PostMapping("/image/upload")
    public CommonResponse<Void> uploadVideoImage(@RequestParam("file") MultipartFile file, @RequestParam Long adVideoId) {
        return adVideoImageService.uploadMuseImage(file, adVideoId);
    }

    @PostMapping("/image/submit")
    public CommonResponse<Void> getAdVideoImageList(@RequestBody AdVideoImageParam adVideoImageParam) {
        return adVideoImageService.updateAdVideoImage(adVideoImageParam);
    }


    @GetMapping("/muse/get")
    public CommonResponse<List<AdMuseImageVo>> getAdMuseImageVoList(@RequestParam Long adVideoId) {
        return adMuseTaskService.getAdMuseImageVoList(adVideoId);
    }

    @GetMapping("/muse/status")
    public CommonResponse<List<AdVideoMuseStatusVo>> getAdVideoMuseStatusVo() {
        List<AdVideoMuseStatusVo> res = new ArrayList<>();
        res.add(new AdVideoMuseStatusVo(AdMuseImageStatusEnum.VERIFY_SUCCESS.getStatus(),
                AdMuseImageStatusEnum.VERIFY_SUCCESS.getDesc()));
        res.add(new AdVideoMuseStatusVo(AdMuseImageStatusEnum.VERIFY_FAIL.getStatus(),
                AdMuseImageStatusEnum.VERIFY_FAIL.getDesc()));
        return CommonResponse.success(res);
    }

    @PostMapping("/muse/update")
    public CommonResponse<Void> updateAdMuseImage(@RequestBody AdMuseImageParam adMuseImageParam) {
        return adMuseTaskService.updateMuseImage(adMuseImageParam);
    }

    @PostMapping("/muse/upload")
    public CommonResponse<Void> uploadMuseImage(@RequestParam("file") MultipartFile file, @RequestParam("id") Long id) {
        return adMuseTaskService.uploadMuseImage(file, id);
    }


}
