package com.tzld.piaoquan.tencentad.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdBaseVideoStatusEnum;
import com.tzld.piaoquan.tencentad.component.HttpPoolClient;
import com.tzld.piaoquan.tencentad.dao.mapper.AdBaseVideoMapper;
import com.tzld.piaoquan.tencentad.model.po.AdBaseVideo;
import com.tzld.piaoquan.tencentad.model.po.AdBaseVideoExample;
import com.tzld.piaoquan.tencentad.model.vo.AdBaseVideoParam;
import com.tzld.piaoquan.tencentad.model.vo.AdBaseVideoVo;
import com.tzld.piaoquan.tencentad.service.AdBaseVideoService;
import com.tzld.piaoquan.tencentad.service.AdVideoService;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import com.tzld.piaoquan.tencentad.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import static com.tzld.piaoquan.tencentad.common.constants.OtherServerURL.POST_VIDEO_DETAIL_URL;

@Slf4j
@Service
public class AdBaseVideoServiceImpl implements AdBaseVideoService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private AdBaseVideoMapper adBaseVideoMapper;

    @Autowired
    private AdVideoService adVideoService;


    @Override
    public CommonResponse<Void> addAdBaseVideo(List<Long> videoIds) {
        if (CollectionUtils.isEmpty(videoIds)) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，添加列表不能为空");
        }
        Map<Long, AdBaseVideo> videoDetails = getVideoDetail(videoIds);
        for (Long videoId : videoIds) {
            AdBaseVideo adBaseVideo = videoDetails.get(videoId);
            if (adBaseVideo == null) {
                continue;
            }
            adBaseVideo.setStatus(AdBaseVideoStatusEnum.FIRST_VERIFY_SUCCESS.getStatus());
            adBaseVideoMapper.insertSelective(adBaseVideo);
        }
        return CommonResponse.success();
    }

    public void refreshBaseVideo() {
        long id = 0L;
        int size = 100;
        List<AdBaseVideo> adBaseVideos;
        do {
            AdBaseVideoExample example = new AdBaseVideoExample();
            example.createCriteria().andIdGreaterThan(id);
            example.setPage(new Page<>(1, size));
            example.setOrderByClause("id asc");
            adBaseVideos = adBaseVideoMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(adBaseVideos)) {
                break;
            }
            id = adBaseVideos.get(adBaseVideos.size() - 1).getId();
            List<Long> videoIds = adBaseVideos.stream().map(AdBaseVideo::getVideoId).collect(Collectors.toList());
            Map<Long, AdBaseVideo> videoDetail = getVideoDetail(videoIds);
            for (AdBaseVideo adBaseVideo : adBaseVideos) {
                AdBaseVideo updateAdBaseVideo = videoDetail.get(adBaseVideo.getVideoId());
                updateAdBaseVideo.setId(adBaseVideo.getId());
                adBaseVideoMapper.updateByPrimaryKeySelective(updateAdBaseVideo);
                adVideoService.updateAdVideo(adBaseVideo);
            }
        } while (!CollectionUtils.isEmpty(adBaseVideos));
    }

    @Override
    public CommonResponse<Page<AdBaseVideoVo>> getAdBaseVideo(Integer pageNum, Integer pageSize,
                                                              List<Integer> status) {
        Page<AdBaseVideoVo> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        AdBaseVideoExample example = new AdBaseVideoExample();
        AdBaseVideoExample.Criteria criteria = example.createCriteria();
        if (status != null) {
            criteria.andStatusIn(status);
        }
        example.setOrderByClause("create_time desc");
        example.setPage(page);
        long total = adBaseVideoMapper.countByExample(example);
        page.setTotalSize((int) total);
        List<AdBaseVideo> adBaseVideos = adBaseVideoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(adBaseVideos)) {
            return CommonResponse.success(page);
        }
        List<AdBaseVideoVo> adBaseVideoVos = new ArrayList<>();
        for (AdBaseVideo adBaseVideo : adBaseVideos) {
            AdBaseVideoVo adBaseVideoVo = new AdBaseVideoVo();
            BeanUtils.copyProperties(adBaseVideo, adBaseVideoVo);
            AdBaseVideoStatusEnum adBaseVideoStatusEnum = AdBaseVideoStatusEnum.getByStatus(adBaseVideo.getStatus());
            if (adBaseVideoStatusEnum != null) {
                adBaseVideoVo.setStatusName(adBaseVideoStatusEnum.getDesc());
            }
            adBaseVideoVo.setCreateTime(DateUtils.getDateString(adBaseVideo.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adBaseVideoVo.setUpdateTime(DateUtils.getDateString(adBaseVideo.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adBaseVideoVos.add(adBaseVideoVo);
        }
        page.setObjs(adBaseVideoVos);
        return CommonResponse.success(page);
    }

    @Override
    public CommonResponse<Void> firstVerifyAdBaseVideo(AdBaseVideoParam adBaseVideoParam) {
        if (adBaseVideoParam == null || adBaseVideoParam.getId() == null || adBaseVideoParam.getStatus() == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
        }
        List<Integer> allowStatus =
                Arrays.asList(AdBaseVideoStatusEnum.FIRST_VERIFY_SUCCESS.getStatus(), AdBaseVideoStatusEnum.FIRST_VERIFY_FAIL.getStatus());
        if (!allowStatus.contains(adBaseVideoParam.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，不允许修改为当前状态");
        }
        AdBaseVideo adBaseVideo = adBaseVideoMapper.selectByPrimaryKey(adBaseVideoParam.getId());
        if (adBaseVideo == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，该视频不存在");
        }
        if (!Objects.equals(adBaseVideo.getStatus(), AdBaseVideoStatusEnum.DEFAULT.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "该状态不允许修改审核状态");
        }
        updateAdBaseVideoStatus(adBaseVideoParam.getId(), adBaseVideoParam.getStatus());
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> secondVerifyAdBaseVideo(AdBaseVideoParam adBaseVideoParam) {
        if (adBaseVideoParam == null || adBaseVideoParam.getId() == null || adBaseVideoParam.getStatus() == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
        }
        List<Integer> allowStatus =
                Arrays.asList(AdBaseVideoStatusEnum.SECOND_VERIFY_SUCCESS.getStatus(), AdBaseVideoStatusEnum.SECOND_VERIFY_FAIL.getStatus());
        if (!allowStatus.contains(adBaseVideoParam.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，不允许修改为当前状态");
        }
        AdBaseVideo adBaseVideo = adBaseVideoMapper.selectByPrimaryKey(adBaseVideoParam.getId());
        if (adBaseVideo == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，该视频不存在");
        }
        if (!Objects.equals(adBaseVideo.getStatus(), AdBaseVideoStatusEnum.FIRST_VERIFY_SUCCESS.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "该状态不允许修改审核状态");
        }
        updateAdBaseVideoStatus(adBaseVideoParam.getId(), adBaseVideoParam.getStatus());
        if (Objects.equals(AdBaseVideoStatusEnum.SECOND_VERIFY_SUCCESS.getStatus(), adBaseVideoParam.getStatus())) {
            adVideoService.addAdVideo(adBaseVideo);
        }
        return CommonResponse.success();
    }

    private void updateAdBaseVideoStatus(Long id, Integer status) {
        AdBaseVideo updateAdBaseVideo = new AdBaseVideo();
        updateAdBaseVideo.setId(id);
        updateAdBaseVideo.setStatus(status);
        adBaseVideoMapper.updateByPrimaryKeySelective(updateAdBaseVideo);
    }

    public Map<Long, AdBaseVideo> getVideoDetail(List<Long> videoIdList) {
        try {
            Map<Long, AdBaseVideo> map = new HashMap<>();
            JSONObject params = new JSONObject();
            params.put("videoIdList", videoIdList);
            String post = httpPoolClient.post(POST_VIDEO_DETAIL_URL, params.toJSONString());
            JSONObject res = JSONObject.parseObject(post);
            JSONArray data = res.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                AdBaseVideo adBaseVideo = new AdBaseVideo();
                JSONObject jsonObject = data.getJSONObject(i);
                Long videoId = jsonObject.getLong("id");
                adBaseVideo.setVideoId(videoId);
                adBaseVideo.setVideoPath(jsonObject.getString("videoPath"));
                adBaseVideo.setVideoCover(jsonObject.getJSONObject("coverImg").getString("coverImgPath"));
                adBaseVideo.setVideoTitle(jsonObject.getString("title"));
                adBaseVideo.setAuditStatus(jsonObject.getInteger("auditStatus"));
                adBaseVideo.setAppAuditStatus(jsonObject.getInteger("appAuditStatus"));
                adBaseVideo.setRecommendStatus(jsonObject.getInteger("recommendStatus"));
                adBaseVideo.setAppRecommendStatus(jsonObject.getInteger("appRecommendStatus"));
                map.put(videoId, adBaseVideo);
            }
            return map;
        } catch (IOException e) {
            log.error("MessageAttachmentServiceImpl getVideDetail error", e);
        }
        return new HashMap<>();
    }
}
